TOOL_DYNAMINE = 'dynamine'
TOOL_DISOMINE = 'disomine'
TOOL_EFOLDMINE = 'efoldmine'
TOOL_AGMATA = 'agmata'
TOOL_PSP = 'psp'

DEPENDENCIES = {
    TOOL_DYNAMINE: [],
    TOOL_DISOMINE: [TOOL_DYNAMINE, TOOL_EFOLDMINE],
    TOOL_EFOLDMINE: [TOOL_DYNAMINE],
    TOOL_AGMATA: [TOOL_DYNAMINE, TOOL_EFOLDMINE],
    TOOL_PSP: []
}